@@ -24,6 +24,7 @@
 
 #include "complain.h"
 #include "files.h"
+#include "fixits.h"
 #include "getargs.h"
 #include "muscle-tab.h"
 #include "quote.h"
@@ -429,12 +430,14 @@ define_directive (char const *assignment,
 /** If the \a variable name is obsolete, return the name to use,
  * otherwise \a variable.  If the \a value is obsolete, update it too.
  *
- * Allocates the returned value.  */
+ * Allocates the returned value if needed, otherwise the returned
+ * value is exactly \a variable.  */
 static
-char *
-muscle_percent_variable_update (char const *variable, location variable_loc,
+char const *
+muscle_percent_variable_update (char const *variable,
                                 muscle_kind kind,
-                                char const **value)
+                                char const **value,
+                                char **old, char **upd)
 {
   typedef struct
   {
@@ -444,8 +447,15 @@ muscle_percent_variable_update (char const *variable, location variable_loc,
   } conversion_type;
   const conversion_type conversion[] =
   {
+    { "%error-verbose",             "parse.error=verbose",       muscle_keyword },
+    { "%error_verbose",             "parse.error=verbose",       muscle_keyword },
+    { "abstract",                   "api.parser.abstract",       muscle_keyword },
+    { "annotations",                "api.parser.annotations",    muscle_code },
     { "api.push_pull",              "api.push-pull",             muscle_keyword },
     { "api.tokens.prefix",          "api.token.prefix",          muscle_code },
+    { "extends",                    "api.parser.extends",        muscle_keyword },
+    { "final",                      "api.parser.final",          muscle_keyword },
+    { "implements",                 "api.parser.implements",     muscle_keyword },
     { "lex_symbol",                 "api.token.constructor",     -1 },
     { "location_type",              "api.location.type",         muscle_code },
     { "lr.default-reductions",      "lr.default-reduction",      muscle_keyword },
@@ -453,16 +463,11 @@ muscle_percent_variable_update (char const *variable, location variable_loc,
     { "lr.keep_unreachable_states", "lr.keep-unreachable-state", muscle_keyword },
     { "namespace",                  "api.namespace",             muscle_code },
     { "parser_class_name",          "api.parser.class",          muscle_code },
+    { "public",                     "api.parser.public",         muscle_keyword },
+    { "strictfp",                   "api.parser.strictfp",       muscle_keyword },
     { "stype",                      "api.value.type",            -1 },
     { "variant=",                   "api.value.type=variant",    -1 },
     { "variant=true",               "api.value.type=variant",    -1 },
-    { "abstract",                   "api.parser.abstract",       muscle_keyword },
-    { "annotations",                "api.parser.annotations",    muscle_code },
-    { "extends",                    "api.parser.extends",        muscle_keyword },
-    { "final",                      "api.parser.final",          muscle_keyword },
-    { "implements",                 "api.parser.implements",     muscle_keyword },
-    { "public",                     "api.parser.public",         muscle_keyword },
-    { "strictfp",                   "api.parser.strictfp",       muscle_keyword },
     { NULL, NULL, -1, }
   };
 
@@ -474,22 +479,25 @@ muscle_percent_variable_update (char const *variable, location variable_loc,
              && STREQ (eq + 1, *value))
           : STREQ (c->obsolete, variable))
         {
-          char *old = define_directive (c->obsolete, kind, *value);
-          char *upd = define_directive (c->updated, c->kind, *value);
-          deprecated_directive (&variable_loc, old, upd);
-          free (old);
-          free (upd);
-          char *res = xstrdup (c->updated);
-          char *eq2 = strchr (res, '=');
-          if (eq2)
-            {
-              *eq2 = '\0';
-              *value = eq2 + 1;
-            }
-          return res;
+          /* Generate the deprecation warning. */
+          *old = c->obsolete[0] == '%'
+            ? xstrdup (c->obsolete)
+            : define_directive (c->obsolete, kind, *value);
+          *upd = define_directive (c->updated, c->kind, *value);
+          /* Update the variable and its value.  */
+          {
+            char *res = xstrdup (c->updated);
+            char *eq2 = strchr (res, '=');
+            if (eq2)
+              {
+                *eq2 = '\0';
+                *value = eq2 + 1;
+              }
+            return res;
+          }
         }
     }
-  return xstrdup (variable);
+  return variable;
 }
 
 void
@@ -499,8 +507,11 @@ muscle_percent_define_insert (char const *var, location variable_loc,
                               muscle_percent_define_how how)
 {
   /* Backward compatibility.  */
-  char *variable
-    = muscle_percent_variable_update (var, variable_loc, kind, &value);
+  char *old = NULL;
+  char *upd = NULL;
+  char const *variable
+    = muscle_percent_variable_update (var, kind,
+                                      &value, &old, &upd);
   uniqstr name = muscle_name (variable, NULL);
   uniqstr loc_name = muscle_name (variable, "loc");
   uniqstr syncline_name = muscle_name (variable, "syncline");
@@ -508,6 +519,7 @@ muscle_percent_define_insert (char const *var, location variable_loc,
   uniqstr kind_name = muscle_name (variable, "kind");
 
   /* Command-line options are processed before the grammar file.  */
+  bool warned = false;
   if (how == MUSCLE_PERCENT_DEFINE_GRAMMAR_FILE
       && muscle_find_const (name))
     {
@@ -521,8 +533,13 @@ muscle_percent_define_insert (char const *var, location variable_loc,
       i += SUB_INDENT;
       location loc = muscle_percent_define_get_loc (variable);
       complain_indent (&loc, complaint, &i, _("previous definition"));
+      fixits_register (&variable_loc, "");
+      warned = true;
     }
 
+  if (!warned && old && upd)
+    deprecated_directive (&variable_loc, old, upd);
+
   MUSCLE_INSERT_STRING (name, value);
   muscle_insert (loc_name, "");
   muscle_location_grow (loc_name, variable_loc);
@@ -533,7 +550,10 @@ muscle_percent_define_insert (char const *var, location variable_loc,
   MUSCLE_INSERT_INT (how_name, how);
   MUSCLE_INSERT_STRING (kind_name, muscle_kind_string (kind));
  end:
-  free (variable);
+  free (old);
+  free (upd);
+  if (variable != var)
+    free ((char *) variable);
 }
 
 /* This is used for backward compatibility, e.g., "%define api.pure"
